# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import unittest

from ecl.tests.functional import base


@unittest.skipUnless(base.service_exists(service_type="metering"),
                     "Metering service does not exist")
class TestStatistics(base.BaseFunctionalTest):

    def test_list(self):
        found_something = False
        for met in self.conn.telemetry.meters():
            try:
                stat = next(self.conn.telemetry.statistics(met))
                self.assertIn('period_end', stat)
                found_something = True
            except Exception:
                pass
        self.assertTrue(found_something)
